"use strict";
var __awaiter = (this && this.__awaiter) || function (thisArg, _arguments, P, generator) {
    return new (P || (P = Promise))(function (resolve, reject) {
        function fulfilled(value) { try { step(generator.next(value)); } catch (e) { reject(e); } }
        function rejected(value) { try { step(generator["throw"](value)); } catch (e) { reject(e); } }
        function step(result) { result.done ? resolve(result.value) : new P(function (resolve) { resolve(result.value); }).then(fulfilled, rejected); }
        step((generator = generator.apply(thisArg, _arguments || [])).next());
    });
};
Object.defineProperty(exports, "__esModule", { value: true });
const n9_node_utils_1 = require("@neo9/n9-node-utils");
const express = require("express");
const helmet = require("helmet");
const http_1 = require("http");
const morgan = require("morgan");
const routing_controllers_1 = require("routing-controllers");
const typedi_1 = require("typedi");
const error_handler_interceptor_1 = require("./middleware/error-handler.interceptor");
const session_loader_interceptor_1 = require("./middleware/session-loader.interceptor");
const requestid_1 = require("./requestid");
function default_1(options) {
    return __awaiter(this, void 0, void 0, function* () {
        // Setup routing-controllers to use typedi container.
        routing_controllers_1.useContainer(typedi_1.Container);
        // Defaults options for routing-controller
        const defaultRoutingControllerOptions = {
            defaults: {
                // with this option, null will return 404 by default
                nullResultCode: 404,
                // with this option, void or Promise<void> will return 204 by default
                undefinedResultCode: 204,
            },
            defaultErrorHandler: false,
            controllers: [options.path + "/**/*.controller.*s"],
        };
        // Default options
        options.http = options.http || {};
        options.http.port = options.http.port || process.env.PORT || 5000;
        options.http.logLevel = (typeof options.http.logLevel !== 'undefined' ? options.http.logLevel : (tokens, req, res) => {
            return [
                options.enableRequestId && res.statusMessage !== 'No Content' ? `(${req.headers.requestId})` : '',
                tokens.method(req, res),
                tokens.url(req, res),
                tokens.status(req, res),
                tokens['response-time'](req, res), 'ms - ',
                tokens.res(req, res, 'content-length')
            ].join(' ');
        });
        options.http.routingController = Object.assign({}, defaultRoutingControllerOptions, options.http.routingController);
        options.http.routingController.interceptors = [session_loader_interceptor_1.SessionLoaderInterceptor, error_handler_interceptor_1.ErrorHandler];
        options.http.routingController.authorizationChecker = (action, roles) => __awaiter(this, void 0, void 0, function* () {
            if (!action.request.headers.session) {
                throw new n9_node_utils_1.N9Error('session-required', 401);
            }
            try {
                action.request.session = JSON.parse(action.request.headers.session);
            }
            catch (err) {
                throw new n9_node_utils_1.N9Error('session-header-is-invalid', 401);
            }
            if (!action.request.session.userId) {
                throw new n9_node_utils_1.N9Error('session-header-has-no-userId', 401);
            }
            return true;
        });
        options.http.routingController.validation = {
            whitelist: true,
            forbidNonWhitelisted: true
        };
        // options.log.info(`-- start-express-app.ts options.http --`, JSON.stringify(options.http, null, 2));
        // Listeners
        const analyzeError = (error) => {
            /* istanbul ignore if */
            if (error.syscall !== 'listen') {
                return error;
            }
            // handle specific listen errors with friendly messages
            switch (error.code) {
                case 'EACCES':
                    return new Error(`Port ${options.http.port} requires elevated privileges`);
                case 'EADDRINUSE':
                    return new Error(`Port ${options.http.port} is already in use`);
                /* istanbul ignore next */
                default:
                    return error;
            }
        };
        const onListening = () => {
            options.log.info('Listening on port ' + options.http.port);
        };
        // Create HTTP server
        let expressApp = express();
        // Middleware
        expressApp.use(requestid_1.setRequestContext);
        expressApp.use(helmet());
        // Logger middleware
        if (options.http.logLevel) {
            expressApp.use(morgan(options.http.logLevel, { stream: options.log.stream }));
        }
        const server = http_1.createServer(expressApp);
        if (options.http.beforeRoutingControllerLaunchHook) {
            yield options.http.beforeRoutingControllerLaunchHook(expressApp, options.log, options);
        }
        expressApp = routing_controllers_1.useExpressServer(expressApp, options.http.routingController);
        if (options.http.afterRoutingControllerLaunchHook) {
            yield options.http.afterRoutingControllerLaunchHook(expressApp, options.log, options);
        }
        // Listen method
        const listen = () => __awaiter(this, void 0, void 0, function* () {
            return new Promise((resolve, reject) => {
                server.listen(options.http.port);
                server.on('error', (error) => {
                    reject(analyzeError(error));
                });
                server.on('listening', () => {
                    onListening();
                    resolve();
                });
            });
        });
        // Make the server listen
        if (!options.http.preventListen)
            yield listen();
        return {
            app: expressApp,
            server
        };
    });
}
exports.default = default_1;
//# sourceMappingURL=start-express-app.js.map